#ifndef lint
static const char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
#endif

#include <stdlib.h>
#include <string.h>

#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYPATCH 20070509

#define YYEMPTY (-1)
#define yyclearin    (yychar = YYEMPTY)
#define yyerrok      (yyerrflag = 0)
#define YYRECOVERING (yyerrflag != 0)

extern int yyparse(void);

static int yygrowstack(void);
#define YYPREFIX "yy"
#line 2 "./config.y"
/*
 * BRLTTY - A background process providing access to the console screen (when in
 *          text mode) for a blind person using a refreshable braille display.
 *
 * Copyright (C) 1995-2009 by The BRLTTY Developers.
 *
 * BRLTTY comes with ABSOLUTELY NO WARRANTY.
 *
 * This is free software, placed under the terms of the
 * GNU General Public License, as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any
 * later version. Please see the file LICENSE-GPL for details.
 *
 * Web Page: http://mielke.cc/brltty/
 *
 * This software is maintained by Dave Mielke <dave@mielke.cc>.
 */

/*
 * Support for all Papenmeier Terminal + config file
 *   Heimo.Schn <heimo.schoen@gmx.at>
 *   August Hrandl <august.hoerandl@gmx.at>
 *
 * Papenmeier/read_config.y
 *  read (scan + interpret) the configuration file - included by braille.c
 *  this file can be used as a standalone test programm - see
 *   Makefile for details
 */

#include "prologue.h"

#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>

#include "brl-cfg.h"

#define YYERROR_VERBOSE
static int yylex (void);
static int yyerror (char*);    
extern int yyparse (void);

/* to be called: */
static int parse (void);

static char *nameval = NULL;
static int numval, keyindex, cmdval; 

static int numkeys = 0;
static int16_t keys[KEYMAX];

static FILE *configurationFile = NULL;
static int lineNumber = 1;

static int terminalsSize;
static int commandsSize;

static int
reallocateTable (void *reference, int size, int count) {
  void **address = reference;
  void *newAddress = realloc(*address, (count * size));
  if (!count || (newAddress != NULL)) {
    *address = newAddress;
    return 1;
  } else {
    yyerror("insufficient memory");
  }
  return 0;
}

static int
allocateTable (void *reference, int size, int count) {
  void **address = reference;
  *address = NULL;
  return reallocateTable(address, size, count);
}

static int
duplicateString (char **address, const char *string) {
  int count = strlen(string) + 1;
  if (!allocateTable(address, 1, count)) return 0;
  memcpy(*address, string, count);
  return 1;
}

static int
ensureTableSize (void *reference, int size, int count, int *limit, int increment) {
  if (count == *limit) {
    int newLimit = *limit + increment;
    if (!reallocateTable(reference, size, newLimit)) return 0;
    *limit = newLimit;
  }
  return 1;
}

static TerminalDefinition *
getCurrentTerminal (void) {
  if (pmTerminalCount > 0) return &pmTerminals[pmTerminalCount - 1];
  yyerror("terminal not being defined");
  return NULL;
}

static int
finishCurrentTerminal (void) {
  if (pmTerminalCount) {
    TerminalDefinition *terminal = &pmTerminals[pmTerminalCount - 1];
    if (!reallocateTable(&terminal->statusCells, sizeof(*terminal->statusCells), terminal->statusCount)) return 0;
    if (!reallocateTable(&terminal->modifierKeys, sizeof(*terminal->modifierKeys), terminal->modifierCount)) return 0;
    if (!reallocateTable(&terminal->commandDefinitions, sizeof(*terminal->commandDefinitions), terminal->commandCount)) return 0;
  }
  return 1;
}

static TerminalDefinition *
addTerminal (int identifier) {
  if (finishCurrentTerminal()) {
    if (ensureTableSize(&pmTerminals, sizeof(*pmTerminals),
                        pmTerminalCount, &terminalsSize, 4)) {
      TerminalDefinition *terminal = &pmTerminals[pmTerminalCount++];
      terminal->modelIdentifier = identifier;
      terminal->modelName = NULL;
      terminal->protocolRevision = 0;
      terminal->helpFile = NULL;

      terminal->textColumns = 0;
      terminal->textRows = 1;

      terminal->frontKeys = 0;
      terminal->hasBar = 0;
      terminal->leftSwitches = 0;
      terminal->rightSwitches = 0;
      terminal->leftKeys = 0;
      terminal->rightKeys = 0;

      terminal->statusCells = NULL;
      terminal->statusCount = 0;

      terminal->modifierKeys = NULL;
      terminal->modifierCount = 0;

      terminal->commandDefinitions = NULL;
      terminal->commandCount = 0;

      commandsSize = 0;
      return terminal;
    }
  }
  return NULL;
}

static int
setName (char *name) {
  TerminalDefinition *terminal = getCurrentTerminal();
  if (terminal) {
    if (!terminal->modelName) {
      if (duplicateString(&terminal->modelName, name)) {
        return 1;
      }
    } else {
      yyerror("duplicate name");
    }
  }
  return 0;
}

static int
setProtocolRevision (int revision) {
  TerminalDefinition *terminal = getCurrentTerminal();
  if (terminal) {
    if ((0 < revision) && (revision <= 0XFF)) {
      if (!terminal->protocolRevision) {
        terminal->protocolRevision = revision;
        return 1;
      } else {
        yyerror("duplicate protocol revision");
      }
    } else {
      yyerror("invalid protocol revision");
    }
  }
  return 0;
}

static int
setHelp (char *file) {
  TerminalDefinition *terminal = getCurrentTerminal();
  if (terminal) {
    if (!terminal->helpFile) {
      if (duplicateString(&terminal->helpFile, file)) {
        return 1;
      }
    } else {
      yyerror("duplicate help file");
    }
  }
  return 0;
}

static int
setColumns (int columns) {
  TerminalDefinition *terminal = getCurrentTerminal();
  if (terminal) {
    if ((0 < columns) && (columns <= 0XFF)) {
      if (!terminal->textColumns) {
        terminal->textColumns = columns;
        return 1;
      } else {
        yyerror("duplicate column count");
      }
    } else {
      yyerror("invalid column count");
    }
  }
  return 0;
}

static int
setStatusCells (int count) {
  TerminalDefinition *terminal = getCurrentTerminal();
  if (terminal) {
    if ((0 < count) && (count <= 0XFF)) {
      if (!terminal->statusCount) {
        if (allocateTable(&terminal->statusCells, sizeof(*terminal->statusCells), count)) {
          {
            int s;
            for (s=0; s<count; s++) terminal->statusCells[s] = OFFS_EMPTY;
          }

          terminal->statusCount = count;
          return 1;
        }
      } else {
        yyerror("duplicate status cell count");
      }
    } else {
      yyerror("invalid status cell count");
    }
  }
  return 0;
}

static int
setFrontKeys (int count) {
  TerminalDefinition *terminal = getCurrentTerminal();
  if (terminal) {
    if ((0 < count) && (count <= 0XFF)) {
      if (!terminal->frontKeys) {
        terminal->frontKeys = count;
        return 1;
      } else {
        yyerror("duplicate front key count");
      }
    } else {
      yyerror("invalid front key count");
    }
  }
  return 0;
}

static int
setHasBar (int ls, int rs, int lk, int rk) {
  TerminalDefinition *terminal = getCurrentTerminal();
  if (terminal) {
    if (terminal->hasBar) {
      yyerror("duplicate bar specification");
    } else if ((ls < 0) || (ls > 1)) {
      yyerror("invalid left switches count");
    } else if ((rs < 0) || (rs > 1)) {
      yyerror("invalid right switches count");
    } else if ((lk < 0) || (lk > 1)) {
      yyerror("invalid left keys count");
    } else if ((rk < 0) || (rk > 1)) {
      yyerror("invalid right keys count");
    } else {
      terminal->hasBar = 1;
      terminal->leftSwitches = ls;
      terminal->rightSwitches = rs;
      terminal->leftKeys = lk;
      terminal->rightKeys = rk;
      return 1;
    }
  }
  return 0;
}

static int
setStatusCell (int number, int code) {
  TerminalDefinition *terminal = getCurrentTerminal();
  if (terminal) {
    if ((0 < number) && (number <= terminal->statusCount)) {
      uint16_t *cell = &terminal->statusCells[number-1];
      if (*cell == OFFS_EMPTY) {
        *cell = code;
        return 1;
      } else {
        yyerror("duplicate status cell");
      }
    } else {
      yyerror("invalid status cell number");
    }
  }
  return 0;
}

static int
addModifier (int key) {
  TerminalDefinition *terminal = getCurrentTerminal();
  if (terminal) {
    {
      int m;
      for (m=0; m<terminal->modifierCount; m++) {
        if (terminal->modifierKeys[m] == key) {
          yyerror("duplicate modifier");
          return 0;
        }
      }
    }

    if (terminal->modifierCount < MODMAX) {
      if (terminal->modifierKeys ||
          allocateTable(&terminal->modifierKeys, sizeof(*terminal->modifierKeys), MODMAX)) {
        terminal->modifierKeys[terminal->modifierCount++] = key;
        return 1;
      }
    } else {
      yyerror("too many modifiers");
    }
  }
  return 0;
}

static CommandDefinition *
addCommand (int code) {
  TerminalDefinition *terminal = getCurrentTerminal();
  if (terminal) {
    if (ensureTableSize(&terminal->commandDefinitions, sizeof(*terminal->commandDefinitions),
                        terminal->commandCount, &commandsSize, 0X20)) {
      CommandDefinition *cmd = &terminal->commandDefinitions[terminal->commandCount++];
      int k;

      cmd->code = code;
      cmd->key = KEY_NONE;
      cmd->modifiers = 0;

      for (k=0; k<numkeys; k++) {
        int16_t key = keys[k];
        int found = 0;
        int m;
        for (m=0; m<terminal->modifierCount; m++) {
          if (key == terminal->modifierKeys[m]) {
            int bit = 1 << m;
            if (cmd->modifiers & bit) {
              yyerror("duplicate modifier");
              goto removeCommand;
            }
            cmd->modifiers |= bit;
            found = 1;
            break;
          }
        }

        if (!found) {
          if (cmd->key != KEY_NONE) {
            yyerror("more than one nonmodifier key");
            goto removeCommand;
          }
          cmd->key = key;
        }
      }
      return cmd;

    removeCommand:
      terminal->commandCount--;
    }
  }
  return NULL;
}
#line 402 "config.tab.c"
#define NUM 257
#define STRING 258
#define IS 259
#define AND 260
#define IDENT 261
#define NAME 262
#define PROTOCOLREVISION 263
#define HELPFILE 264
#define DISPLAYSIZE 265
#define STATCELLS 266
#define FRONTKEYS 267
#define HASBAR 268
#define MODIFIER 269
#define ROUTING1 270
#define ROUTING2 271
#define BAR 272
#define SWITCH 273
#define KEY 274
#define LEFT 275
#define RIGHT 276
#define REAR 277
#define FRONT 278
#define STAT 279
#define STATUS2 280
#define KEYCODE 281
#define STATCODE 282
#define BARCODE 283
#define NUMBER 284
#define FLAG 285
#define HORIZ 286
#define ON 287
#define OFF 288
#define CHECK 289
#define ROUTE 290
#define VPK 291
#define YYERRCODE 256
short yylhs[] = {                                        -1,
    0,    0,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    2,    2,    2,    8,    8,    8,    3,    6,    6,    4,
    4,    4,    4,    5,    5,    5,    5,    5,    5,    5,
    5,    5,    5,    5,    5,    5,    5,    7,    7,    9,
};
short yylen[] = {                                         2,
    0,    2,    1,    2,    4,    4,    4,    4,    4,    4,
    4,    2,    7,    4,    4,    4,    5,    5,    5,    5,
    1,    1,    0,    1,    1,    0,    2,    1,    1,    1,
    2,    2,    2,    2,    2,    2,    3,    3,    3,    3,
    3,    3,    3,    3,    1,    1,    1,    1,    3,    1,
};
short yydefred[] = {                                      1,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   28,   29,    3,    2,    0,    0,    4,   22,
   21,    0,    0,    0,    0,    0,    0,    0,   12,    0,
    0,   27,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   45,   46,    0,    0,
    0,    0,    0,   47,    0,   30,    0,    0,    0,    0,
    0,    0,    0,    0,   50,    0,    0,    5,    6,    7,
    8,    9,   10,   11,    0,   36,    0,    0,    0,    0,
   35,   34,   15,   33,   32,   31,   14,    0,    0,    0,
    0,   16,   25,   24,    0,    0,   37,   38,   39,   40,
   41,   42,   43,   44,   17,   18,   19,   20,   49,    0,
   13,
};
short yydgoto[] = {                                       1,
   16,   22,   17,   60,   65,   18,   66,   95,   67,
};
short yysindex[] = {                                      0,
  -10,   16,  -56,  -56,  -56,  -56,  -56,  -56,  -56,   -9,
  -56, -226,    0,    0,    0,    0,  -56,  -59,    0,    0,
    0, -221, -248, -197, -191, -189, -188, -187,    0, -186,
 -251,    0, -252,  -56,  -56,  -56,  -56, -251,   62,   63,
   64,   65,   66,   67,   68, -178,    0,    0, -203, -219,
 -217, -176, -175,    0,   73,    0, -198, -196, -195,   75,
 -251, -251, -251, -251,    0,   78,  -40,    0,    0,    0,
    0,    0,    0,    0, -168,    0, -227, -216, -214, -212,
    0,    0,    0,    0,    0,    0,    0,   80,   81,   82,
   83,    0,    0,    0, -251, -163,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   85,
    0,
};
short yyrindex[] = {                                      0,
    0,    0, -161, -160, -161, -160, -161, -161, -161, -161,
 -225,    0,    0,    0,    0,    0, -247, -225,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, -225, -225, -225, -225,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   -6,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,
};
short yygindex[] = {                                      0,
    0,    7,    0,    0,   69,    0,  -55,    0,    0,
};
#define YYTABLESIZE 281
short yytable[] = {                                      15,
   29,   21,   94,   48,   21,   88,   89,   90,   91,   40,
   23,   24,   25,   26,   27,   28,   30,   31,   47,   48,
   49,   50,   51,   33,   38,   19,   52,   53,   54,   56,
   32,   57,   58,   59,   23,   39,   23,   23,   23,  109,
   61,   62,   63,   64,   23,   23,   23,   23,   23,   97,
   98,   21,   23,   23,   23,   77,   78,   79,   80,   41,
   99,  100,  101,  102,  103,  104,   42,   43,   44,   45,
   46,   68,   69,   70,   71,   72,   73,   74,   75,   76,
   81,   82,   83,   84,   87,   85,   86,   92,   96,  105,
  106,  107,  108,  110,  111,   23,    0,   23,    0,   55,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   20,
    0,    0,   20,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   93,
    0,    0,    0,    0,    0,    0,    0,   34,   35,   36,
   37,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    2,    0,    0,    0,   20,
    3,    4,    5,    6,    7,    8,    9,   10,   11,    0,
    0,    0,    0,   26,   26,   26,   26,   26,   12,    0,
   13,   26,   26,   26,    0,    0,    0,    0,    0,    0,
   14,
};
short yycheck[] = {                                      10,
   10,   61,   43,   10,   61,   61,   62,   63,   64,  258,
    4,    5,    6,    7,    8,    9,   10,   11,  270,  271,
  272,  273,  274,   17,   18,   10,  278,  279,  280,  282,
  257,  284,  285,  286,  282,  257,  284,  285,  286,   95,
   34,   35,   36,   37,  270,  271,  272,  273,  274,  277,
  278,   61,  278,  279,  280,  275,  276,  275,  276,  257,
  277,  278,  277,  278,  277,  278,  258,  257,  257,  257,
  257,   10,   10,   10,   10,   10,   10,   10,  257,  283,
  257,  257,   10,  282,   10,  282,  282,   10,  257,   10,
   10,   10,   10,  257,   10,  257,   -1,  258,   -1,   31,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  259,
   -1,   -1,  259,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  260,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  287,  288,  289,
  290,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  256,   -1,   -1,   -1,  259,
  261,  262,  263,  264,  265,  266,  267,  268,  269,   -1,
   -1,   -1,   -1,  270,  271,  272,  273,  274,  279,   -1,
  281,  278,  279,  280,   -1,   -1,   -1,   -1,   -1,   -1,
  291,
};
#define YYFINAL 1
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 291
#if YYDEBUG
char *yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,"'\\n'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,"'+'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'='",0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"NUM","STRING","IS",
"AND","IDENT","NAME","PROTOCOLREVISION","HELPFILE","DISPLAYSIZE","STATCELLS",
"FRONTKEYS","HASBAR","MODIFIER","ROUTING1","ROUTING2","BAR","SWITCH","KEY",
"LEFT","RIGHT","REAR","FRONT","STAT","STATUS2","KEYCODE","STATCODE","BARCODE",
"NUMBER","FLAG","HORIZ","ON","OFF","CHECK","ROUTE","VPK",
};
char *yyrule[] = {
"$accept : input",
"input :",
"input : input inputline",
"inputline : '\\n'",
"inputline : error '\\n'",
"inputline : IDENT eq NUM '\\n'",
"inputline : NAME eq STRING '\\n'",
"inputline : PROTOCOLREVISION eq NUM '\\n'",
"inputline : HELPFILE eq STRING '\\n'",
"inputline : DISPLAYSIZE eq NUM '\\n'",
"inputline : STATCELLS eq NUM '\\n'",
"inputline : FRONTKEYS eq NUM '\\n'",
"inputline : HASBAR '\\n'",
"inputline : HASBAR eq NUM NUM NUM NUM '\\n'",
"inputline : statdef eq statdisp '\\n'",
"inputline : MODIFIER eq anykey '\\n'",
"inputline : keycode eq modifiers '\\n'",
"inputline : keycode ON eq modifiers '\\n'",
"inputline : keycode OFF eq modifiers '\\n'",
"inputline : keycode CHECK eq modifiers '\\n'",
"inputline : keycode ROUTE eq modifiers '\\n'",
"eq : '='",
"eq : IS",
"eq :",
"and : '+'",
"and : AND",
"and :",
"statdef : STAT NUM",
"keycode : KEYCODE",
"keycode : VPK",
"statdisp : STATCODE",
"statdisp : HORIZ STATCODE",
"statdisp : FLAG STATCODE",
"statdisp : NUMBER STATCODE",
"anykey : STAT NUM",
"anykey : FRONT NUM",
"anykey : BAR BARCODE",
"anykey : SWITCH LEFT REAR",
"anykey : SWITCH LEFT FRONT",
"anykey : SWITCH RIGHT REAR",
"anykey : SWITCH RIGHT FRONT",
"anykey : KEY LEFT REAR",
"anykey : KEY LEFT FRONT",
"anykey : KEY RIGHT REAR",
"anykey : KEY RIGHT FRONT",
"anykey : ROUTING1",
"anykey : ROUTING2",
"anykey : STATUS2",
"modifiers : modifier",
"modifiers : modifier and modifiers",
"modifier : anykey",
};
#endif
#ifndef YYSTYPE
typedef int YYSTYPE;
#endif
#if YYDEBUG
#include <stdio.h>
#endif

/* define the initial stack-sizes */
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH  YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH  10000
#endif
#endif

#define YYINITSTACKSIZE 500

int      yydebug;
int      yynerrs;
int      yyerrflag;
int      yychar;
short   *yyssp;
YYSTYPE *yyvsp;
YYSTYPE  yyval;
YYSTYPE  yylval;

/* variables for the parser stack */
static short   *yyss;
static short   *yysslim;
static YYSTYPE *yyvs;
static int      yystacksize;
#line 467 "./config.y"

/* --------------------------------------------------------------- */
/* all the keywords */
/* the commands CMD_* CR_* in brl.h are autogenerated - see Makefile */

struct init_v
{
  char*  sname;			/* symbol (lowercase) */
  int token;			/* yacc token */
  int val;			/* key value */
};

static struct init_v symbols[]= {
  { "helpfile",    HELPFILE ,0 },
  { "is",          IS, 0 },
  { "and",         AND, 0 },

  { "identification", IDENT, 0 },
  { "identity",    IDENT, 0 },
  { "terminal",    NAME, 0 },
  { "protocolrevision", PROTOCOLREVISION, 0 },
  { "type",        NAME, 0 },
  { "typ",         NAME, 0 },

  { "modifier",    MODIFIER, 0},

  { "displaysize", DISPLAYSIZE, 0 },
  { "statuscells", STATCELLS, 0 },
  { "frontkeys",   FRONTKEYS, 0 },
  { "hasbar",      HASBAR,    0 },

  { "status",      STAT,    0 },
  { "status2",     STATUS2, 0 },
  { "bar",         BAR,     0 },

  { "front",       FRONT,  0 },
  { "rear",        REAR,   0 },
  { "left",        LEFT,   0 },
  { "right",       RIGHT,  0 },

  { "switch",      SWITCH, 0 },
  { "key",         KEY,    0 },

  { "horiz",       HORIZ,  0 },
  { "flag",        FLAG,   0 },
  { "number",      NUMBER, 0 },

  { "left1",       BARCODE, BAR_L1},
  { "left2",       BARCODE, BAR_L2 },
  { "up1",         BARCODE, BAR_U1},
  { "up2",         BARCODE, BAR_U2 },
  { "right1",      BARCODE, BAR_R1},
  { "right2",      BARCODE, BAR_R2 },
  { "down1",       BARCODE, BAR_D1},
  { "down2",       BARCODE, BAR_D2 },

  { "routing1",    ROUTING1, 0 },
  { "routing2",    ROUTING2, 0 },

  { "on",          ON, 0 },
  { "off",         OFF, 0 },
  { "check",       CHECK, 0 },
  { "route",       ROUTE, 0 },

#include "cmd.auto.h"
  { "EOF", KEYCODE, -1 },
  { NULL, 0, 0 }
};

/* --------------------------------------------------------------- */
/* all the help */
/* the commands CMD_* in brl.h are autogenerated - see Makefile */

struct help_v
{
  int    cmd;			/* cmd */
  char*  txt;			/* help text */
};

static struct help_v hlptxt[]= {
#include "hlp.auto.h"
  { 0, NULL }
};


/* --------------------------------------------------------------- */

int yylex ()
{
  int c;
  static char symbuf[40] = { 0 };
  const int length = ARRAY_COUNT(symbuf);

  /* Ignore whitespace, get first nonwhite character.  */
  while ((c = getc(configurationFile)) == ' ' || c == '\t')
    ;

  if (c == EOF)
    return 0;

  if (c == '#') {		/* comment to end of line */
    do {
      c = getc(configurationFile);
    } while (c != '\n' && c != EOF);
    lineNumber++;
    return '\n';
  }

  /* Char starts a number => parse the number. */
  if (c == '.' || isdigit (c)) {
    ungetc (c, configurationFile);
    fscanf (configurationFile, "%d", &numval);
    yylval = numval;
    return NUM;
  }

  if (c == '"') {		/* string */
    int i=0;
    symbuf[0] = '\0';
    c = getc(configurationFile);
    while(c !='"' && c != EOF) {
      /* If buffer is full */
      if (i == length)
	break;
      /* Add this character to the buffer. */
      symbuf[i++] = c;
      c = getc(configurationFile);
    }
    symbuf[i] = 0;
    nameval = symbuf;
    return STRING;
  }

  /* Char starts an identifier => read the name. */
  if (isalpha (c) || c=='_') {
    int i = 0;
    do {
      /* If buffer is full */
      if (i == length)
	break;
      
      /* Add this character to the buffer. */
      symbuf[i++] = c;
      c = getc(configurationFile);
    } while (c != EOF && (isalnum (c) || c=='_'));
     
    ungetc (c, configurationFile);
    symbuf[i] = 0;
    for(i=0;symbols[i].sname; i++)
      if(strcasecmp(symbuf,symbols[i].sname) == 0) {
	numval = symbols[i].val;
	return symbols[i].token;
      }
    /* unused feature */
    nameval = symbuf;
    return NAME;
  }
     
  /* Any other character is a token by itself. */
  if (c == '\n')
    lineNumber++;

  return c;
}


/* --------------------------------------------------------------- */

void
deallocateTerminalTable (void) {
  while (pmTerminalCount) {
    TerminalDefinition *terminal = &pmTerminals[--pmTerminalCount];
    if (terminal->modelName) free(terminal->modelName);
    if (terminal->helpFile) free(terminal->helpFile);
    if (terminal->statusCells) free(terminal->statusCells);
    if (terminal->modifierKeys) free(terminal->modifierKeys);
    if (terminal->commandDefinitions) free(terminal->commandDefinitions);
  }

  if (pmTerminals) {
    free(pmTerminals);
    pmTerminals = NULL;
  }
}

int
parseConfigurationFile (void) {
  if (pmTerminalsAllocated) {
    deallocateTerminalTable();
  } else {
    pmTerminalCount = 0;
    pmTerminals = NULL;
    pmTerminalsAllocated = 1;
  }

  lineNumber = 1;
  terminalsSize = 0;

  nameval = NULL;
  numval = 0;
  keyindex = 0;
  cmdval = 0;
  numkeys = 0;

  {
    int result = yyparse();
    finishCurrentTerminal();
    return result;
  }
}
#line 884 "config.tab.c"
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(void)
{
    int newsize, i;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;

    i = yyssp - yyss;
    newss = (yyss != 0)
          ? (short *)realloc(yyss, newsize * sizeof(*newss))
          : (short *)malloc(newsize * sizeof(*newss));
    if (newss == 0)
        return -1;

    yyss  = newss;
    yyssp = newss + i;
    newvs = (yyvs != 0)
          ? (YYSTYPE *)realloc(yyvs, newsize * sizeof(*newvs))
          : (YYSTYPE *)malloc(newsize * sizeof(*newvs));
    if (newvs == 0)
        return -1;

    yyvs = newvs;
    yyvsp = newvs + i;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab
int
yyparse(void)
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register const char *yys;

    if ((yys = getenv("YYDEBUG")) != 0)
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = YYEMPTY;

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = YYEMPTY;
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;

    yyerror("syntax error");

#ifdef lint
    goto yyerrlab;
#endif

yyerrlab:
    ++yynerrs;

yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = YYEMPTY;
        goto yyloop;
    }

yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    if (yym)
        yyval = yyvsp[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);
    switch (yyn)
    {
case 4:
#line 400 "./config.y"
{ yyerrok;  }
break;
case 5:
#line 401 "./config.y"
{ addTerminal(numval); }
break;
case 6:
#line 402 "./config.y"
{ setName(nameval); }
break;
case 7:
#line 403 "./config.y"
{ setProtocolRevision(numval); }
break;
case 8:
#line 404 "./config.y"
{ setHelp(nameval); }
break;
case 9:
#line 405 "./config.y"
{ setColumns(numval); }
break;
case 10:
#line 406 "./config.y"
{ setStatusCells(numval); }
break;
case 11:
#line 407 "./config.y"
{ setFrontKeys(numval); }
break;
case 12:
#line 408 "./config.y"
{ setHasBar(1, 1, 1, 1); }
break;
case 13:
#line 409 "./config.y"
{ setHasBar(yyvsp[-4], yyvsp[-3], yyvsp[-2], yyvsp[-1]); }
break;
case 14:
#line 411 "./config.y"
{ setStatusCell(keyindex, numval);  }
break;
case 15:
#line 412 "./config.y"
{ addModifier(keyindex); }
break;
case 16:
#line 413 "./config.y"
{ addCommand(cmdval); }
break;
case 17:
#line 414 "./config.y"
{ addCommand(cmdval | BRL_FLG_TOGGLE_ON); }
break;
case 18:
#line 415 "./config.y"
{ addCommand(cmdval | BRL_FLG_TOGGLE_OFF); }
break;
case 19:
#line 416 "./config.y"
{ addCommand(cmdval | BRL_FLG_TOGGLE_ON | BRL_FLG_TOGGLE_OFF); }
break;
case 20:
#line 417 "./config.y"
{ addCommand(cmdval | BRL_FLG_ROUTE); }
break;
case 27:
#line 430 "./config.y"
{ keyindex=numval;  }
break;
case 28:
#line 433 "./config.y"
{ cmdval=numval; numkeys = 0; }
break;
case 29:
#line 434 "./config.y"
{ cmdval = numval; numkeys = 0; }
break;
case 30:
#line 437 "./config.y"
{  }
break;
case 31:
#line 438 "./config.y"
{ numval += OFFS_HORIZ; }
break;
case 32:
#line 439 "./config.y"
{ numval += OFFS_FLAG; }
break;
case 33:
#line 440 "./config.y"
{ numval += OFFS_NUMBER; }
break;
case 34:
#line 443 "./config.y"
{ keyindex= KEYS_STATUS + numval; }
break;
case 35:
#line 444 "./config.y"
{ keyindex= KEYS_FRONT + numval; }
break;
case 36:
#line 445 "./config.y"
{ keyindex= KEYS_BAR + numval; }
break;
case 37:
#line 446 "./config.y"
{ keyindex= KEYS_SWITCH + SWITCH_LEFT_REAR; }
break;
case 38:
#line 447 "./config.y"
{ keyindex= KEYS_SWITCH + SWITCH_LEFT_FRONT; }
break;
case 39:
#line 448 "./config.y"
{ keyindex= KEYS_SWITCH + SWITCH_RIGHT_REAR; }
break;
case 40:
#line 449 "./config.y"
{ keyindex= KEYS_SWITCH + SWITCH_RIGHT_FRONT; }
break;
case 41:
#line 450 "./config.y"
{ keyindex= KEYS_SWITCH + KEY_LEFT_REAR; }
break;
case 42:
#line 451 "./config.y"
{ keyindex= KEYS_SWITCH + KEY_LEFT_FRONT; }
break;
case 43:
#line 452 "./config.y"
{ keyindex= KEYS_SWITCH + KEY_RIGHT_REAR; }
break;
case 44:
#line 453 "./config.y"
{ keyindex= KEYS_SWITCH + KEY_RIGHT_FRONT; }
break;
case 45:
#line 454 "./config.y"
{ keyindex= KEY_ROUTING1; }
break;
case 46:
#line 455 "./config.y"
{ keyindex= KEY_ROUTING2; }
break;
case 47:
#line 456 "./config.y"
{ keyindex= KEY_STATUS2; }
break;
case 50:
#line 463 "./config.y"
{ keys[numkeys++] = keyindex; }
break;
#line 1221 "config.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;

yyoverflow:
    yyerror("yacc stack overflow");

yyabort:
    return (1);

yyaccept:
    return (0);
}
